
package com.redxun.knowledge.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.db.PageHelper;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.model.JPaasUser;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.dto.user.OsUserDto;
import com.redxun.knowledge.common.MessageService;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.Knowledge;
import com.redxun.knowledge.entity.dao.KnowledgeApprove;
import com.redxun.knowledge.entity.vo.KnowledgeApprovalListVo;
import com.redxun.knowledge.entity.vo.KnowledgeCategoryAdminVo;
import com.redxun.knowledge.mapper.KnowledgeApproveMapper;
import com.redxun.knowledge.mapper.KnowledgeMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
* [业务--知识审批表]业务服务类
*/
@Service
@Slf4j
public class KnowledgeApproveServiceImpl extends SuperServiceImpl<KnowledgeApproveMapper, KnowledgeApprove> implements BaseService<KnowledgeApprove> {

    @Resource
    private KnowledgeApproveMapper knowledgeApproveMapper;

    @Autowired
    private KnowledgeCategoryServiceImpl knowledgeCategoryService;

    @Autowired
    private KnowledgeMapper knowledgeMapper;

    @Autowired
    private MessageService messageService;

    @Autowired
    private UserService userService;

    @Override
    public BaseDao<KnowledgeApprove> getRepository() {
        return knowledgeApproveMapper;
    }

    /**
     * 审核管理列表(分页查询)
     * @param filter
     * @return
     */
    public IPage findAllKnowledgeApprove(QueryFilter filter) {
        Map<String, Object> params = PageHelper.constructParams(filter);
        //查询出全部记录
        List<KnowledgeApprovalListVo> allKnowledgeApprove = knowledgeApproveMapper.findAllKnowledgeApprove(filter.getParams(), params);
        // 根据知识Id进行数据分组
        Map<String, List<KnowledgeApprovalListVo>> result =
                allKnowledgeApprove.stream().collect(Collectors.groupingBy(KnowledgeApprovalListVo::getKnowledgerId));
        // 结果集
        List<KnowledgeApprovalListVo> knowledgeApprovalListVoList = new ArrayList<>();
        // 遍历每组的数据
        Set<Map.Entry<String, List<KnowledgeApprovalListVo>>> entries = result.entrySet();
        for (Map.Entry<String, List<KnowledgeApprovalListVo>> entry : entries) {
            List<KnowledgeApprovalListVo> value = entry.getValue();
            for (int i = 0; i < value.size(); i++) {
                // 如果是最终审核通过,直接放入记录
                if (value.get(i).getApprovalStatus() == 5 && value.get(i).getIsFinal() == 1){
                    knowledgeApprovalListVoList.add(value.get(i));
                    break;
                }
                // 判断审批结果是否为null(如果审批结果没有null，证明审批已结束;如果等于0，证明审批没通过)
                if (value.get(i).getResult() == null || value.get(i).getResult() == 0) {
                    if (i != 0) {
                        // 如果不是第一条数据,取前一条数据的审批时间
                        Date approvalTime = value.get(i - 1).getApprovalTime();
                        value.get(i).setApprovalTime(approvalTime);
                    }
                    // 把该记录加到结果集中
                    knowledgeApprovalListVoList.add(value.get(i));
                    // 如果节点为终审节点，审核状态为3(待最终审核)
                    if (value.get(i).getIsFinal() == 1 && value.get(i).getApprovalStatus() != 5){
                        knowledgeMapper.updateApprovalStatus(value.get(i).getKnowledgerId(),3);
                    }
                    break;
                }
            }
        }
        // 放入知识分类信息
        List<KnowledgeApprovalListVo> collect = knowledgeApprovalListVoList.stream().
                sorted((t1, t2) -> t2.getCreateTime().compareTo(t1.getCreateTime())).collect(Collectors.toList());
        collect.forEach(e -> {
            String categoryId = e.getCategoryId();
            KnowledgeCategoryAdminVo knowledgeCategoryAdminVo = knowledgeCategoryService.findAllParentByCategoryId(categoryId);
            e.setKnowledgeCategoryAdminVo(knowledgeCategoryAdminVo);
        });
        // 重新组装分页数据
        IPage pages = filter.getPage();
        pages.setRecords(collect);
        pages.setTotal(collect.size());
        return pages;
    }

    /**
     * 知识审核
     * @param entity
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int update(KnowledgeApprove entity) {
        KnowledgeApprove knowledgeApprove = knowledgeApproveMapper.selectById(entity.getPkId());
        // 如果审核节点不是终审节点且知识未通过,修改知识状态为节点驳回 2
        if (knowledgeApprove.getIsFinal() == 0 && entity.getResult() == 0){
            Knowledge knowledge = new Knowledge();
            knowledge.setPkId(knowledgeApprove.getKnowledgerId());
            knowledge.setApprovalStatus(2);
            knowledgeMapper.updateById(knowledge);
            //给知识创建人发送站内信和企业微信消息
            String createBy = knowledgeMapper.selectById(knowledgeApprove.getKnowledgerId()).getCreateBy();
            sendMessage(createBy,entity.getRemark());
        }
        // 如果审核节点是终审节点且知识未通过,修改知识状态为最终驳回 4
        if (knowledgeApprove.getIsFinal() == 1 && entity.getResult() == 0){
            Knowledge knowledge = new Knowledge();
            knowledge.setPkId(knowledgeApprove.getKnowledgerId());
            knowledge.setApprovalStatus(4);
            knowledgeMapper.updateById(knowledge);
            //给知识创建人发送站内信和企业微信消息
            String createBy = knowledgeMapper.selectById(knowledgeApprove.getKnowledgerId()).getCreateBy();
            sendMessage(createBy,entity.getRemark());
        }
        // 如果审核节点是终审节点且知识通过,修改知识状态为最终通过 5
        if (knowledgeApprove.getIsFinal() == 1 && entity.getResult() == 1){
            Knowledge knowledge = new Knowledge();
            knowledge.setPkId(knowledgeApprove.getKnowledgerId());
            knowledge.setApprovalStatus(5);
            knowledgeMapper.updateById(knowledge);
            //给知识创建人发送站内信和企业微信消息
            String createBy = knowledgeMapper.selectById(knowledgeApprove.getKnowledgerId()).getCreateBy();
            sendMessage(createBy,entity.getRemark());
        }
        // 如果审核通过了,发送短信通知下一个人审核
        if (knowledgeApprove.getResult() == 1 && knowledgeApprove.getIsFinal() != 1){
            // TODO: 2022/12/9
        }
        return this.getRepository().updateById(entity);
    }

    /**
     * 给知识创建人发送站内信和企业微信消息(知识被驳回时,或者最终节点审核通过时)
     * @param createBy,remark
     */
    private void sendMessage(String createBy,String remark){
        List<OsUserDto> osUserDtoList = new ArrayList<>();
        //查询用户信息
        OsUserDto osUserDto = userService.queryOsUserDto(createBy);
        osUserDtoList.add(osUserDto);
        if (remark != null){
            messageService.sendWechatMsg("[关于知识审核不通过通知]",remark,osUserDtoList);
            messageService.sendInnerMsg("[关于知识审核不通过通知]",remark,osUserDtoList);
        } else {
            messageService.sendWechatMsg("[关于知识审核不通过通知]","知识未通过",osUserDtoList);
            messageService.sendInnerMsg("[关于知识审核不通过通知]","知识未通过",osUserDtoList);
        }
    }
}
